@@ -199,6 +199,41 @@ class StorageApp(Flask):
 
         return new_handler
 
+    def _list_resource_only(self, f: Callable):
+        """
+        Restricts this handler to only refer list resources.
+        MUST be used below _using_resource, directly or not.
+        :param f: The handler function to invoke.
+        :return: A new handler which also checks the proper type
+          of the resource: list.
+        """
+
+        @functools.wraps(f)
+        def new_handler(resource: str, *args, **kwargs):
+            resource_definition = self._settings["resources"].get(resource)
+            if resource_definition["type"] != "list":
+                return not_found()
+            return f(resource, *args, **kwargs)
+        return new_handler
+
+    def _simple_resource_only(self, f: Callable):
+        """
+        Restricts this handler to only refer list resources.
+        MUST be used below _using_resource, directly or not.
+        :param f: The handler function to invoke.
+        :return: A new handler which also checks the proper type
+          of the resource: simple.
+        """
+
+        @functools.wraps(f)
+        def new_handler(resource: str, *args, **kwargs):
+            resource_definition = self._settings["resources"].get(resource)
+            if resource_definition["type"] != "simple":
+                return not_found()
+
+            return f(resource, *args, **kwargs)
+        return new_handler
+
     def _prepare_indexes(self):
         """
         Prepares the indices for each setup.
@@ -399,6 +434,7 @@ class StorageApp(Flask):
         @self.route("/<string:resource>/~<string:method>", methods=["GET", "POST"])
         @self._capture_unexpected_errors
         @self._using_resource
+        @self._simple_resource_only
         @self._bearer_required
         def resource_method(resource: str, resource_definition: dict, db_name: str, collection_name: str,
                             collection: Collection, filter: dict, method: str):
@@ -430,6 +466,7 @@ class StorageApp(Flask):
         @self.route("/<string:resource>", methods=["PUT"])
         @self._capture_unexpected_errors
         @self._using_resource
+        @self._simple_resource_only
         @self._bearer_required
         def resource_replace(resource: str, resource_definition: dict, db_name: str, collection_name: str,
                              collection: Collection, filter: dict):
@@ -471,6 +508,7 @@ class StorageApp(Flask):
         @self.route("/<string:resource>", methods=["PATCH"])
         @self._capture_unexpected_errors
         @self._using_resource
+        @self._simple_resource_only
         @self._bearer_required
         def resource_update(resource: str, resource_definition: dict, db_name: str, collection_name: str,
                             collection: Collection, filter: dict):
@@ -512,6 +550,7 @@ class StorageApp(Flask):
         @self.route("/<string:resource>", methods=["DELETE"])
         @self._capture_unexpected_errors
         @self._using_resource
+        @self._simple_resource_only
         @self._bearer_required
         def resource_delete(resource: str, resource_definition: dict, db_name: str, collection_name: str,
                             collection: Collection, filter: dict):
@@ -542,6 +581,7 @@ class StorageApp(Flask):
         @self.route("/<string:resource>/<regex('[a-f0-9]{24}'):object_id>", methods=["GET"])
         @self._capture_unexpected_errors
         @self._using_resource
+        @self._list_resource_only
         @self._bearer_required
         def item_resource_read(resource: str, resource_definition: dict, db_name: str, collection_name: str,
                                collection: Collection, filter: dict, object_id: str):
@@ -564,6 +604,7 @@ class StorageApp(Flask):
         @self.route("/<string:resource>/<regex('[a-f0-9]{24}'):object_id>", methods=["PUT"])
         @self._capture_unexpected_errors
         @self._using_resource
+        @self._list_resource_only
         @self._bearer_required
         def item_resource_replace(resource: str, resource_definition: dict, db_name: str, collection_name: str,
                                   collection: Collection, filter: dict, object_id: str):
@@ -605,6 +646,7 @@ class StorageApp(Flask):
         @self.route("/<string:resource>/<regex('[a-f0-9]{24}'):object_id>", methods=["PATCH"])
         @self._capture_unexpected_errors
         @self._using_resource
+        @self._list_resource_only
         @self._bearer_required
         def item_resource_update(resource: str, resource_definition: dict, db_name: str, collection_name: str,
                                  collection: Collection, filter: dict, object_id: str):
@@ -646,6 +688,7 @@ class StorageApp(Flask):
         @self.route("/<string:resource>/<regex('[a-f0-9]{24}'):object_id>", methods=["DELETE"])
         @self._capture_unexpected_errors
         @self._using_resource
+        @self._list_resource_only
         @self._bearer_required
         def item_resource_delete(resource: str, resource_definition: dict, db_name: str, collection_name: str,
                                  collection: Collection, filter: dict, object_id: str):
@@ -675,6 +718,7 @@ class StorageApp(Flask):
         @self.route("/<string:resource>/<regex('[a-f0-9]{24}'):object_id>/~<string:method>", methods=["GET", "POST"])
         @self._capture_unexpected_errors
         @self._using_resource
+        @self._list_resource_only
         @self._bearer_required
         def item_resource_method(resource: str, resource_definition: dict, db_name: str, collection_name: str,
                                  collection: Collection, filter: dict, object_id: str, method: str):
